#include "FAtoRRGConverter.h"

#include <alphabet/LabeledSymbol.h>

namespace fa2rg
{

grammar::RightRG FAtoRRGConverter::convert(const automaton::NFA& automaton)
{
    std::map<automaton::State, alphabet::Symbol> nonterminalMap;

    const automaton::State& initState = *automaton.getInitialStates().begin();
    const alphabet::Symbol initSymbol(alphabet::LabeledSymbol(initState.getName()));

    grammar::RightRG grammar(initSymbol);
    nonterminalMap.insert(std::make_pair(initState, initSymbol));

    grammar.setTerminalAlphabet(automaton.getInputAlphabet());

    for(const auto& state : automaton.getStates())
    {
        if(state == initState)
            continue;

        alphabet::Symbol nt = alphabet::createUniqueSymbol(alphabet::Symbol(alphabet::LabeledSymbol(state.getName())), grammar.getTerminalAlphabet(), grammar.getNonterminalAlphabet());
        grammar.addNonterminalSymbol(nt);
        nonterminalMap.insert(std::make_pair(state, nt));
    }

    // step 2 - create set of P in G
    for(const auto& transition : automaton.getTransitions())
    {
        const automaton::State& from = transition.first.first;
        const alphabet::Symbol& input = transition.first.second;
        for(const auto& to : transition.second)
        {
            grammar.addRule(nonterminalMap.find(from)->second, std::make_pair(input, nonterminalMap.find(to)->second)); // 2a
            if(automaton.getFinalStates().count(to)) // 2b
                grammar.addRule(nonterminalMap.find(from)->second, input);
        }
    }

    // step 3 - set start symbol of G
    grammar.setInitialSymbol(nonterminalMap.find(*automaton.getInitialStates().begin())->second);

    // step 4
    if(automaton.getFinalStates().count(*automaton.getInitialStates().begin()))
    {
        if( ! this->isSymbolOnAnyRightHandSide(grammar.getInitialSymbol(), grammar)) // 4a
        {
            grammar.setGeneratesEpsilon(true);
        }
        else // 4b
        {
            alphabet::Symbol newStart = alphabet::createUniqueSymbol(grammar.getInitialSymbol(), grammar.getTerminalAlphabet(), grammar.getNonterminalAlphabet());
            grammar.addNonterminalSymbol(newStart);

            for(const auto& rule : grammar.getRules())
                for(const auto& rightSide : rule.second)
                    if(rule.first == grammar.getInitialSymbol())
                        grammar.addRule(newStart, rightSide);

            grammar.setInitialSymbol(newStart);
            grammar.setGeneratesEpsilon(true);
        }
    }

    return grammar;
}

bool FAtoRRGConverter::isSymbolOnAnyRightHandSide(const alphabet::Symbol& symbol, const grammar::RightRG& grammar) const
{
    for(const auto& rule : grammar.getRules())
        for(const auto& rightSide : rule.second)
            if(rightSide.is<std::pair<alphabet::Symbol, alphabet::Symbol>>( ))
            {
                const std::pair<alphabet::Symbol, alphabet::Symbol>& rhs = rightSide.get<std::pair<alphabet::Symbol, alphabet::Symbol>>();
                if(rhs.first == symbol || rhs.second == symbol)
                    return true;
            }
            else
            {
                const alphabet::Symbol& rhs = rightSide.get<alphabet::Symbol>();
                if(rhs == symbol)
                    return true;
            }

    return false;
}

} /* namespace fa2rg */
