#include <list>
#include "StringTest.h"

#include "sax/SaxParseInterface.h"
#include "sax/SaxComposeInterface.h"

#include "string/String.h"
#include "string/LinearString.h"
#include "string/CyclicString.h"
#include "string/Epsilon.h"
#include "string/StringFromStringParser.h"
#include "string/StringToStringComposer.h"
#include "string/StringFromXMLParser.h"
#include "string/StringToXMLComposer.h"

#include "factory/DataFactory.hpp"

#include "alphabet/Symbol.h"
#include "alphabet/LabeledSymbol.h"

#include "label/StringLabel.h"
#include "label/IntegerLabel.h"
#include "label/Label.h"

#define CPPUNIT_IMPLY(x, y) CPPUNIT_ASSERT(!(x) || (y))

CPPUNIT_TEST_SUITE_REGISTRATION( StringTest );

void StringTest::setUp() {
}

void StringTest::tearDown() {
}

void StringTest::testCopyConstruct() {
  string::LinearString linearString;
  linearString.setAlphabet({alphabet::Symbol(alphabet::LabeledSymbol(label::Label(label::StringLabel("1")))), alphabet::Symbol(alphabet::LabeledSymbol(label::Label(label::StringLabel("2")))), alphabet::Symbol(alphabet::LabeledSymbol(label::Label(label::StringLabel("3"))))});
  linearString.setContent({
			  alphabet::Symbol(alphabet::LabeledSymbol(label::Label(label::StringLabel("1")))),
			  alphabet::Symbol(alphabet::LabeledSymbol(label::Label(label::StringLabel("2")))),
			  alphabet::Symbol(alphabet::LabeledSymbol(label::Label(label::StringLabel("1"))))
			  });
  string::String string(linearString);
  string::String string2(string);
  
  CPPUNIT_ASSERT( string == string2 );
  
  string::String string3(std::move(string));
  
  CPPUNIT_ASSERT( string2 == string3 );
}

void StringTest::testEqual() {
  {
    std::string input = "\"aa\"";
    std::stringstream inputs(input);

    string::StringFromStringParser parser(inputs);
    string::String string = parser.parseValue();

    string::StringToStringComposer composer;
    std::string output = composer.compose(string);
    std::stringstream outputs(output);

    CPPUNIT_ASSERT( input == output );

    string::StringFromStringParser parser2(outputs);
    string::String string2 = parser2.parseValue();

    CPPUNIT_ASSERT( string == string2 );
  }
  {
    std::string input = "\"'aaaa''aaa'\"";
    std::stringstream inputs(input);

    string::StringFromStringParser parser(inputs);
    string::String string = parser.parseValue();

    string::StringToStringComposer composer;
    std::string output = composer.compose(string);
    std::stringstream outputs(output);

    CPPUNIT_ASSERT( input == output );

    string::StringFromStringParser parser2(outputs);
    string::String string2 = parser2.parseValue();

    CPPUNIT_ASSERT( string == string2 );
  }
  {
    std::string input = "#E";
    std::stringstream inputs(input);

    string::StringFromStringParser parser(inputs);
    string::String string = parser.parseValue();

    string::StringToStringComposer composer;
    std::string output = composer.compose(string);
    std::stringstream outputs(output);

    CPPUNIT_ASSERT( input == output );

    string::StringFromStringParser parser2(outputs);
    string::String string2 = parser2.parseValue();

    CPPUNIT_ASSERT( string == string2 );
  }
}

void StringTest::testXMLParser() {

  string::LinearString linearString;
  linearString.setAlphabet({alphabet::Symbol(alphabet::LabeledSymbol(label::Label(label::StringLabel("1")))), alphabet::Symbol(alphabet::LabeledSymbol(label::Label(label::StringLabel("2")))), alphabet::Symbol(alphabet::LabeledSymbol(label::Label(label::StringLabel("3"))))});
  linearString.setContent({alphabet::Symbol(alphabet::LabeledSymbol(label::Label(label::StringLabel("1")))),
					alphabet::Symbol(alphabet::LabeledSymbol(label::Label(label::StringLabel("2")))),
					alphabet::Symbol(alphabet::LabeledSymbol(label::Label(label::StringLabel("1"))))}
		  );

  string::String string(linearString);
  {
	std::list<sax::Token> tokens = alib::DataFactory::toTokens(string);
	std::string tmp;
	sax::SaxComposeInterface::printMemory(tmp, tokens);

	std::list<sax::Token> tokens2;
	sax::SaxParseInterface::parseMemory(tmp, tokens2);
	string::String string2 = alib::DataFactory::fromTokens<string::String>(tokens2);

	CPPUNIT_ASSERT( string == string2 );
  }
  {
	std::string tmp = alib::DataFactory::toString(string);
	string::String string2 = alib::DataFactory::fromString<string::String>(tmp);

	CPPUNIT_ASSERT( string == string2 );
  }
}

void StringTest::testCompare() {
  string::String linear(string::LinearString {});
  string::String cyclic(string::CyclicString {});
  string::String epsilon(string::Epsilon {});

  CPPUNIT_ASSERT(linear == epsilon);
  CPPUNIT_ASSERT(cyclic == epsilon);
  CPPUNIT_ASSERT(cyclic == linear);
  CPPUNIT_ASSERT(epsilon == linear);
  CPPUNIT_ASSERT(epsilon == cyclic);
  CPPUNIT_ASSERT(linear == cyclic);
}

void StringTest::testCompareCyclic() {
  string::CyclicString cyclic1({alphabet::Symbol(alphabet::LabeledSymbol(label::Label(label::IntegerLabel(0)))), alphabet::Symbol(alphabet::LabeledSymbol(label::Label(label::IntegerLabel(1)))), alphabet::Symbol(alphabet::LabeledSymbol(label::Label(label::IntegerLabel(2))))});
  string::CyclicString cyclic2({alphabet::Symbol(alphabet::LabeledSymbol(label::Label(label::IntegerLabel(1)))), alphabet::Symbol(alphabet::LabeledSymbol(label::Label(label::IntegerLabel(2)))), alphabet::Symbol(alphabet::LabeledSymbol(label::Label(label::IntegerLabel(0))))});
  string::CyclicString cyclic3({alphabet::Symbol(alphabet::LabeledSymbol(label::Label(label::IntegerLabel(2)))), alphabet::Symbol(alphabet::LabeledSymbol(label::Label(label::IntegerLabel(0)))), alphabet::Symbol(alphabet::LabeledSymbol(label::Label(label::IntegerLabel(1))))});

  string::CyclicString cyclic4({alphabet::Symbol(alphabet::LabeledSymbol(label::Label(label::IntegerLabel(2)))), alphabet::Symbol(alphabet::LabeledSymbol(label::Label(label::IntegerLabel(1)))), alphabet::Symbol(alphabet::LabeledSymbol(label::Label(label::IntegerLabel(0))))});

  CPPUNIT_ASSERT(cyclic1 == cyclic2);
  CPPUNIT_ASSERT(cyclic1 == cyclic3);
  CPPUNIT_ASSERT(cyclic2 == cyclic3);

  CPPUNIT_ASSERT(cyclic4 != cyclic1);
  CPPUNIT_ASSERT(cyclic4 != cyclic2);
  CPPUNIT_ASSERT(cyclic4 != cyclic3);
}
